# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2020-05-19.      #
#                                                           #
# Ruby Bindings Version 2.1.26                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # 20x4 character alphanumeric display with blue backlight
  class BrickletLCD20x4 < Device
    DEVICE_IDENTIFIER = 212 # :nodoc:
    DEVICE_DISPLAY_NAME = 'LCD 20x4 Bricklet' # :nodoc:

    # This callback is triggered when a button is pressed. The parameter is
    # the number of the button (0 to 2 or 0 to 3 since hardware version 1.2).
    CALLBACK_BUTTON_PRESSED = 9

    # This callback is triggered when a button is released. The parameter is
    # the number of the button (0 to 2 or 0 to 3 since hardware version 1.2).
    CALLBACK_BUTTON_RELEASED = 10

    FUNCTION_WRITE_LINE = 1 # :nodoc:
    FUNCTION_CLEAR_DISPLAY = 2 # :nodoc:
    FUNCTION_BACKLIGHT_ON = 3 # :nodoc:
    FUNCTION_BACKLIGHT_OFF = 4 # :nodoc:
    FUNCTION_IS_BACKLIGHT_ON = 5 # :nodoc:
    FUNCTION_SET_CONFIG = 6 # :nodoc:
    FUNCTION_GET_CONFIG = 7 # :nodoc:
    FUNCTION_IS_BUTTON_PRESSED = 8 # :nodoc:
    FUNCTION_SET_CUSTOM_CHARACTER = 11 # :nodoc:
    FUNCTION_GET_CUSTOM_CHARACTER = 12 # :nodoc:
    FUNCTION_SET_DEFAULT_TEXT = 13 # :nodoc:
    FUNCTION_GET_DEFAULT_TEXT = 14 # :nodoc:
    FUNCTION_SET_DEFAULT_TEXT_COUNTER = 15 # :nodoc:
    FUNCTION_GET_DEFAULT_TEXT_COUNTER = 16 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:


    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 2]

      @response_expected[FUNCTION_WRITE_LINE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_CLEAR_DISPLAY] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_BACKLIGHT_ON] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_BACKLIGHT_OFF] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_IS_BACKLIGHT_ON] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_CONFIG] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_CONFIG] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_IS_BUTTON_PRESSED] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_CUSTOM_CHARACTER] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_CUSTOM_CHARACTER] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_DEFAULT_TEXT] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_DEFAULT_TEXT] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_DEFAULT_TEXT_COUNTER] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_DEFAULT_TEXT_COUNTER] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

      @callback_formats[CALLBACK_BUTTON_PRESSED] = [9, 'C']
      @callback_formats[CALLBACK_BUTTON_RELEASED] = [9, 'C']

      @ipcon.add_device self
    end

    # Writes text to a specific line with a specific position.
    # The text can have a maximum of 20 characters.
    # 
    # For example: (0, 7, "Hello") will write *Hello* in the middle of the
    # first line of the display.
    # 
    # The display uses a special charset that includes all ASCII characters except
    # backslash and tilde. The LCD charset also includes several other non-ASCII characters, see
    # the `charset specification <https://github.com/Tinkerforge/lcd-20x4-bricklet/raw/master/datasheets/standard_charset.pdf>`__
    # for details. The Unicode example above shows how to specify non-ASCII characters
    # and how to translate from Unicode to the LCD charset.
    def write_line(line, position, text)
      check_validity

      send_request FUNCTION_WRITE_LINE, [line, position, text], 'C C Z20', 8, ''
    end

    # Deletes all characters from the display.
    def clear_display
      check_validity

      send_request FUNCTION_CLEAR_DISPLAY, [], '', 8, ''
    end

    # Turns the backlight on.
    def backlight_on
      check_validity

      send_request FUNCTION_BACKLIGHT_ON, [], '', 8, ''
    end

    # Turns the backlight off.
    def backlight_off
      check_validity

      send_request FUNCTION_BACKLIGHT_OFF, [], '', 8, ''
    end

    # Returns *true* if the backlight is on and *false* otherwise.
    def is_backlight_on
      check_validity

      send_request FUNCTION_IS_BACKLIGHT_ON, [], '', 9, '?'
    end

    # Configures if the cursor (shown as "_") should be visible and if it
    # should be blinking (shown as a blinking block). The cursor position
    # is one character behind the the last text written with
    # BrickletLCD20x4#write_line.
    def set_config(cursor, blinking)
      check_validity

      send_request FUNCTION_SET_CONFIG, [cursor, blinking], '? ?', 8, ''
    end

    # Returns the configuration as set by BrickletLCD20x4#set_config.
    def get_config
      check_validity

      send_request FUNCTION_GET_CONFIG, [], '', 10, '? ?'
    end

    # Returns *true* if the button (0 to 2 or 0 to 3 since hardware version 1.2)
    # is pressed.
    # 
    # If you want to react on button presses and releases it is recommended to use
    # the CALLBACK_BUTTON_PRESSED and CALLBACK_BUTTON_RELEASED callbacks.
    def is_button_pressed(button)
      check_validity

      send_request FUNCTION_IS_BUTTON_PRESSED, [button], 'C', 9, '?'
    end

    # The LCD 20x4 Bricklet can store up to 8 custom characters. The characters
    # consist of 5x8 pixels and can be addressed with the index 0-7. To describe
    # the pixels, the first 5 bits of 8 bytes are used. For example, to make
    # a custom character "H", you should transfer the following:
    # 
    # * ``character[0] = 0b00010001`` (decimal value 17)
    # * ``character[1] = 0b00010001`` (decimal value 17)
    # * ``character[2] = 0b00010001`` (decimal value 17)
    # * ``character[3] = 0b00011111`` (decimal value 31)
    # * ``character[4] = 0b00010001`` (decimal value 17)
    # * ``character[5] = 0b00010001`` (decimal value 17)
    # * ``character[6] = 0b00010001`` (decimal value 17)
    # * ``character[7] = 0b00000000`` (decimal value 0)
    # 
    # The characters can later be written with BrickletLCD20x4#write_line by using the
    # characters with the byte representation 8 ("\\x08" or "\\u0008") to 15
    # ("\\x0F" or "\\u000F").
    # 
    # You can play around with the custom characters in Brick Viewer version
    # since 2.0.1.
    # 
    # Custom characters are stored by the LCD in RAM, so they have to be set
    # after each startup.
    # 
    # .. versionadded:: 2.0.1$nbsp;(Plugin)
    def set_custom_character(index, character)
      check_validity

      send_request FUNCTION_SET_CUSTOM_CHARACTER, [index, character], 'C C8', 8, ''
    end

    # Returns the custom character for a given index, as set with
    # BrickletLCD20x4#set_custom_character.
    # 
    # .. versionadded:: 2.0.1$nbsp;(Plugin)
    def get_custom_character(index)
      check_validity

      send_request FUNCTION_GET_CUSTOM_CHARACTER, [index], 'C', 16, 'C8'
    end

    # Sets the default text for lines 0-3. The max number of characters
    # per line is 20.
    # 
    # The default text is shown on the LCD, if the default text counter
    # expires, see BrickletLCD20x4#set_default_text_counter.
    # 
    # .. versionadded:: 2.0.2$nbsp;(Plugin)
    def set_default_text(line, text)
      check_validity

      send_request FUNCTION_SET_DEFAULT_TEXT, [line, text], 'C Z20', 8, ''
    end

    # Returns the default text for a given line (0-3) as set by
    # BrickletLCD20x4#set_default_text.
    # 
    # .. versionadded:: 2.0.2$nbsp;(Plugin)
    def get_default_text(line)
      check_validity

      send_request FUNCTION_GET_DEFAULT_TEXT, [line], 'C', 28, 'Z20'
    end

    # Sets the default text counter. This counter is decremented each
    # ms by the LCD firmware. If the counter reaches 0, the default text
    # (see BrickletLCD20x4#set_default_text) is shown on the LCD.
    # 
    # This functionality can be used to show a default text if the controlling
    # program crashes or the connection is interrupted.
    # 
    # A possible approach is to call BrickletLCD20x4#set_default_text_counter every
    # minute with the parameter 1000*60*2 (2 minutes). In this case the
    # default text will be shown no later than 2 minutes after the
    # controlling program crashes.
    # 
    # A negative counter turns the default text functionality off.
    # 
    # .. versionadded:: 2.0.2$nbsp;(Plugin)
    def set_default_text_counter(counter)
      check_validity

      send_request FUNCTION_SET_DEFAULT_TEXT_COUNTER, [counter], 'l', 8, ''
    end

    # Returns the current value of the default text counter.
    # 
    # .. versionadded:: 2.0.2$nbsp;(Plugin)
    def get_default_text_counter
      check_validity

      send_request FUNCTION_GET_DEFAULT_TEXT_COUNTER, [], '', 12, 'l'
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # The Raspberry Pi HAT (Zero) Brick is always at position 'i' and the Bricklet
    # connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always as
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

    # Registers a callback with ID <tt>id</tt> to the block <tt>block</tt>.
    def register_callback(id, &block)
      callback = block
      @registered_callbacks[id] = callback
    end
  end
end
