# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2020-05-19.      #
#                                                           #
# Ruby Bindings Version 2.1.26                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # Capacitive touch sensor for 12 electrodes
  class BrickletMultiTouch < Device
    DEVICE_IDENTIFIER = 234 # :nodoc:
    DEVICE_DISPLAY_NAME = 'Multi Touch Bricklet' # :nodoc:

    # Returns the current touch state, see BrickletMultiTouch#get_touch_state for
    # information about the state.
    # 
    # This callback is triggered every time the touch state changes.
    CALLBACK_TOUCH_STATE = 5

    FUNCTION_GET_TOUCH_STATE = 1 # :nodoc:
    FUNCTION_RECALIBRATE = 2 # :nodoc:
    FUNCTION_SET_ELECTRODE_CONFIG = 3 # :nodoc:
    FUNCTION_GET_ELECTRODE_CONFIG = 4 # :nodoc:
    FUNCTION_SET_ELECTRODE_SENSITIVITY = 6 # :nodoc:
    FUNCTION_GET_ELECTRODE_SENSITIVITY = 7 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:


    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 0]

      @response_expected[FUNCTION_GET_TOUCH_STATE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_RECALIBRATE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_SET_ELECTRODE_CONFIG] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_ELECTRODE_CONFIG] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_ELECTRODE_SENSITIVITY] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_ELECTRODE_SENSITIVITY] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

      @callback_formats[CALLBACK_TOUCH_STATE] = [10, 'S']

      @ipcon.add_device self
    end

    # Returns the current touch state. The state is given as a bitfield.
    # 
    # Bits 0 to 11 represent the 12 electrodes and bit 12 represents
    # the proximity.
    # 
    # If an electrode is touched, the corresponding bit is *true*. If
    # a hand or similar is in proximity to the electrodes, bit 12 is
    # *true*.
    # 
    # Example: The state 4103 = 0x1007 = 0b1000000000111 means that
    # electrodes 0, 1 and 2 are touched and that something is in the
    # proximity of the electrodes.
    # 
    # The proximity is activated with a distance of 1-2cm. An electrode
    # is already counted as touched if a finger is nearly touching the
    # electrode. This means that you can put a piece of paper or foil
    # or similar on top of a electrode to build a touch panel with
    # a professional look.
    def get_touch_state
      check_validity

      send_request FUNCTION_GET_TOUCH_STATE, [], '', 10, 'S'
    end

    # Recalibrates the electrodes. Call this function whenever you changed
    # or moved you electrodes.
    def recalibrate
      check_validity

      send_request FUNCTION_RECALIBRATE, [], '', 8, ''
    end

    # Enables/disables electrodes with a bitfield (see BrickletMultiTouch#get_touch_state).
    # 
    # *True* enables the electrode, *false* disables the electrode. A
    # disabled electrode will always return *false* as its state. If you
    # don't need all electrodes you can disable the electrodes that are
    # not needed.
    # 
    # It is recommended that you disable the proximity bit (bit 12) if
    # the proximity feature is not needed. This will reduce the amount of
    # traffic that is produced by the CALLBACK_TOUCH_STATE callback.
    # 
    # Disabling electrodes will also reduce power consumption.
    # 
    # Default: 8191 = 0x1FFF = 0b1111111111111 (all electrodes and proximity feature enabled)
    def set_electrode_config(enabled_electrodes)
      check_validity

      send_request FUNCTION_SET_ELECTRODE_CONFIG, [enabled_electrodes], 'S', 8, ''
    end

    # Returns the electrode configuration, as set by BrickletMultiTouch#set_electrode_config.
    def get_electrode_config
      check_validity

      send_request FUNCTION_GET_ELECTRODE_CONFIG, [], '', 10, 'S'
    end

    # Sets the sensitivity of the electrodes. An electrode with a high sensitivity
    # will register a touch earlier then an electrode with a low sensitivity.
    # 
    # If you build a big electrode you might need to decrease the sensitivity, since
    # the area that can be charged will get bigger. If you want to be able to
    # activate an electrode from further away you need to increase the sensitivity.
    # 
    # After a new sensitivity is set, you likely want to call BrickletMultiTouch#recalibrate
    # to calibrate the electrodes with the newly defined sensitivity.
    def set_electrode_sensitivity(sensitivity)
      check_validity

      send_request FUNCTION_SET_ELECTRODE_SENSITIVITY, [sensitivity], 'C', 8, ''
    end

    # Returns the current sensitivity, as set by BrickletMultiTouch#set_electrode_sensitivity.
    def get_electrode_sensitivity
      check_validity

      send_request FUNCTION_GET_ELECTRODE_SENSITIVITY, [], '', 9, 'C'
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # The Raspberry Pi HAT (Zero) Brick is always at position 'i' and the Bricklet
    # connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always as
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

    # Registers a callback with ID <tt>id</tt> to the block <tt>block</tt>.
    def register_callback(id, &block)
      callback = block
      @registered_callbacks[id] = callback
    end
  end
end
