# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2020-05-19.      #
#                                                           #
# Ruby Bindings Version 2.1.26                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # 3.3cm (1.3") OLED display with 128x64 pixels
  class BrickletOLED128x64 < Device
    DEVICE_IDENTIFIER = 263 # :nodoc:
    DEVICE_DISPLAY_NAME = 'OLED 128x64 Bricklet' # :nodoc:

    FUNCTION_WRITE = 1 # :nodoc:
    FUNCTION_NEW_WINDOW = 2 # :nodoc:
    FUNCTION_CLEAR_DISPLAY = 3 # :nodoc:
    FUNCTION_SET_DISPLAY_CONFIGURATION = 4 # :nodoc:
    FUNCTION_GET_DISPLAY_CONFIGURATION = 5 # :nodoc:
    FUNCTION_WRITE_LINE = 6 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:


    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 0]

      @response_expected[FUNCTION_WRITE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_NEW_WINDOW] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_CLEAR_DISPLAY] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_SET_DISPLAY_CONFIGURATION] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_DISPLAY_CONFIGURATION] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_WRITE_LINE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE


      @ipcon.add_device self
    end

    # Appends 64 byte of data to the window as set by BrickletOLED128x64#new_window.
    # 
    # Each row has a height of 8 pixels which corresponds to one byte of data.
    # 
    # Example: if you call BrickletOLED128x64#new_window with column from 0 to 127 and row
    # from 0 to 7 (the whole display) each call of BrickletOLED128x64#write (red arrow) will
    # write half of a row.
    # 
    # .. image:: /Images/Bricklets/bricklet_oled_128x64_display.png
    #    :scale: 100 %
    #    :alt: Display pixel order
    #    :align: center
    #    :target: ../../_images/Bricklets/bricklet_oled_128x64_display.png
    # 
    # The LSB (D0) of each data byte is at the top and the MSB (D7) is at the
    # bottom of the row.
    # 
    # The next call of BrickletOLED128x64#write will write the second half of the row
    # and the next two the second row and so on. To fill the whole display
    # you need to call BrickletOLED128x64#write 16 times.
    def write(data)
      check_validity

      send_request FUNCTION_WRITE, [data], 'C64', 8, ''
    end

    # Sets the window in which you can write with BrickletOLED128x64#write. One row
    # has a height of 8 pixels.
    def new_window(column_from, column_to, row_from, row_to)
      check_validity

      send_request FUNCTION_NEW_WINDOW, [column_from, column_to, row_from, row_to], 'C C C C', 8, ''
    end

    # Clears the current content of the window as set by BrickletOLED128x64#new_window.
    def clear_display
      check_validity

      send_request FUNCTION_CLEAR_DISPLAY, [], '', 8, ''
    end

    # Sets the configuration of the display.
    # 
    # You can set a contrast value from 0 to 255 and you can invert the color
    # (black/white) of the display.
    def set_display_configuration(contrast, invert)
      check_validity

      send_request FUNCTION_SET_DISPLAY_CONFIGURATION, [contrast, invert], 'C ?', 8, ''
    end

    # Returns the configuration as set by BrickletOLED128x64#set_display_configuration.
    def get_display_configuration
      check_validity

      send_request FUNCTION_GET_DISPLAY_CONFIGURATION, [], '', 10, 'C ?'
    end

    # Writes text to a specific line with a specific position.
    # The text can have a maximum of 26 characters.
    # 
    # For example: (1, 10, "Hello") will write *Hello* in the middle of the
    # second line of the display.
    # 
    # You can draw to the display with BrickletOLED128x64#write and then add text to it
    # afterwards.
    # 
    # The display uses a special 5x7 pixel charset. You can view the characters
    # of the charset in Brick Viewer.
    def write_line(line, position, text)
      check_validity

      send_request FUNCTION_WRITE_LINE, [line, position, text], 'C C Z26', 8, ''
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # The Raspberry Pi HAT (Zero) Brick is always at position 'i' and the Bricklet
    # connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always as
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

  end
end
