# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2020-05-19.      #
#                                                           #
# Ruby Bindings Version 2.1.26                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # Controls AC and DC Solid State Relays
  class BrickletSolidStateRelay < Device
    DEVICE_IDENTIFIER = 244 # :nodoc:
    DEVICE_DISPLAY_NAME = 'Solid State Relay Bricklet' # :nodoc:

    # This callback is triggered whenever the monoflop timer reaches 0.
    # The parameter is the current state of the relay
    # (the state after the monoflop).
    CALLBACK_MONOFLOP_DONE = 5

    FUNCTION_SET_STATE = 1 # :nodoc:
    FUNCTION_GET_STATE = 2 # :nodoc:
    FUNCTION_SET_MONOFLOP = 3 # :nodoc:
    FUNCTION_GET_MONOFLOP = 4 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:


    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 0]

      @response_expected[FUNCTION_SET_STATE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_STATE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_MONOFLOP] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_MONOFLOP] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

      @callback_formats[CALLBACK_MONOFLOP_DONE] = [9, '?']

      @ipcon.add_device self
    end

    # Sets the state of the relays *true* means on and *false* means off.
    # 
    # A running monoflop timer will be aborted if this function is called.
    def set_state(state)
      check_validity

      send_request FUNCTION_SET_STATE, [state], '?', 8, ''
    end

    # Returns the state of the relay, *true* means on and *false* means off.
    def get_state
      check_validity

      send_request FUNCTION_GET_STATE, [], '', 9, '?'
    end

    # The first parameter  is the desired state of the relay (*true* means on
    # and *false* means off). The second parameter indicates the time that
    # the relay should hold the state.
    # 
    # If this function is called with the parameters (true, 1500):
    # The relay will turn on and in 1.5s it will turn off again.
    # 
    # A monoflop can be used as a failsafe mechanism. For example: Lets assume you
    # have a RS485 bus and a Solid State Relay Bricklet connected to one of the slave
    # stacks. You can now call this function every second, with a time parameter
    # of two seconds. The relay will be on all the time. If now the RS485
    # connection is lost, the relay will turn off in at most two seconds.
    def set_monoflop(state, time)
      check_validity

      send_request FUNCTION_SET_MONOFLOP, [state, time], '? L', 8, ''
    end

    # Returns the current state and the time as set by
    # BrickletSolidStateRelay#set_monoflop as well as the remaining time until the state flips.
    # 
    # If the timer is not running currently, the remaining time will be returned
    # as 0.
    def get_monoflop
      check_validity

      send_request FUNCTION_GET_MONOFLOP, [], '', 17, '? L L'
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # The Raspberry Pi HAT (Zero) Brick is always at position 'i' and the Bricklet
    # connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always as
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

    # Registers a callback with ID <tt>id</tt> to the block <tt>block</tt>.
    def register_callback(id, &block)
      callback = block
      @registered_callbacks[id] = callback
    end
  end
end
