# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2020-05-19.      #
#                                                           #
# Ruby Bindings Version 2.1.26                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # Measures ambient temperature with 0.5°C accuracy
  class BrickletTemperature < Device
    DEVICE_IDENTIFIER = 216 # :nodoc:
    DEVICE_DISPLAY_NAME = 'Temperature Bricklet' # :nodoc:

    # This callback is triggered periodically with the period that is set by
    # BrickletTemperature#set_temperature_callback_period. The parameter is the
    # temperature of the sensor.
    # 
    # The CALLBACK_TEMPERATURE callback is only triggered if the temperature has changed
    # since the last triggering.
    CALLBACK_TEMPERATURE = 8

    # This callback is triggered when the threshold as set by
    # BrickletTemperature#set_temperature_callback_threshold is reached.
    # The parameter is the temperature of the sensor.
    # 
    # If the threshold keeps being reached, the callback is triggered periodically
    # with the period as set by BrickletTemperature#set_debounce_period.
    CALLBACK_TEMPERATURE_REACHED = 9

    FUNCTION_GET_TEMPERATURE = 1 # :nodoc:
    FUNCTION_SET_TEMPERATURE_CALLBACK_PERIOD = 2 # :nodoc:
    FUNCTION_GET_TEMPERATURE_CALLBACK_PERIOD = 3 # :nodoc:
    FUNCTION_SET_TEMPERATURE_CALLBACK_THRESHOLD = 4 # :nodoc:
    FUNCTION_GET_TEMPERATURE_CALLBACK_THRESHOLD = 5 # :nodoc:
    FUNCTION_SET_DEBOUNCE_PERIOD = 6 # :nodoc:
    FUNCTION_GET_DEBOUNCE_PERIOD = 7 # :nodoc:
    FUNCTION_SET_I2C_MODE = 10 # :nodoc:
    FUNCTION_GET_I2C_MODE = 11 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:

    THRESHOLD_OPTION_OFF = 'x' # :nodoc:
    THRESHOLD_OPTION_OUTSIDE = 'o' # :nodoc:
    THRESHOLD_OPTION_INSIDE = 'i' # :nodoc:
    THRESHOLD_OPTION_SMALLER = '<' # :nodoc:
    THRESHOLD_OPTION_GREATER = '>' # :nodoc:
    I2C_MODE_FAST = 0 # :nodoc:
    I2C_MODE_SLOW = 1 # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 1]

      @response_expected[FUNCTION_GET_TEMPERATURE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_TEMPERATURE_CALLBACK_PERIOD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_TEMPERATURE_CALLBACK_PERIOD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_TEMPERATURE_CALLBACK_THRESHOLD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_TEMPERATURE_CALLBACK_THRESHOLD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_DEBOUNCE_PERIOD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_DEBOUNCE_PERIOD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_I2C_MODE] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_I2C_MODE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

      @callback_formats[CALLBACK_TEMPERATURE] = [10, 's']
      @callback_formats[CALLBACK_TEMPERATURE_REACHED] = [10, 's']

      @ipcon.add_device self
    end

    # Returns the temperature of the sensor.
    # 
    # If you want to get the temperature periodically, it is recommended
    # to use the CALLBACK_TEMPERATURE callback and set the period with
    # BrickletTemperature#set_temperature_callback_period.
    def get_temperature
      check_validity

      send_request FUNCTION_GET_TEMPERATURE, [], '', 10, 's'
    end

    # Sets the period with which the CALLBACK_TEMPERATURE callback is triggered
    # periodically. A value of 0 turns the callback off.
    # 
    # The CALLBACK_TEMPERATURE callback is only triggered if the temperature has changed
    # since the last triggering.
    def set_temperature_callback_period(period)
      check_validity

      send_request FUNCTION_SET_TEMPERATURE_CALLBACK_PERIOD, [period], 'L', 8, ''
    end

    # Returns the period as set by BrickletTemperature#set_temperature_callback_period.
    def get_temperature_callback_period
      check_validity

      send_request FUNCTION_GET_TEMPERATURE_CALLBACK_PERIOD, [], '', 12, 'L'
    end

    # Sets the thresholds for the CALLBACK_TEMPERATURE_REACHED callback.
    # 
    # The following options are possible:
    # 
    #  "Option", "Description"
    # 
    #  "'x'",    "Callback is turned off"
    #  "'o'",    "Callback is triggered when the temperature is *outside* the min and max values"
    #  "'i'",    "Callback is triggered when the temperature is *inside* the min and max values"
    #  "'<'",    "Callback is triggered when the temperature is smaller than the min value (max is ignored)"
    #  "'>'",    "Callback is triggered when the temperature is greater than the min value (max is ignored)"
    def set_temperature_callback_threshold(option, min, max)
      check_validity

      send_request FUNCTION_SET_TEMPERATURE_CALLBACK_THRESHOLD, [option, min, max], 'k s s', 8, ''
    end

    # Returns the threshold as set by BrickletTemperature#set_temperature_callback_threshold.
    def get_temperature_callback_threshold
      check_validity

      send_request FUNCTION_GET_TEMPERATURE_CALLBACK_THRESHOLD, [], '', 13, 'k s s'
    end

    # Sets the period with which the threshold callback
    # 
    # * CALLBACK_TEMPERATURE_REACHED
    # 
    # is triggered, if the threshold
    # 
    # * BrickletTemperature#set_temperature_callback_threshold
    # 
    # keeps being reached.
    def set_debounce_period(debounce)
      check_validity

      send_request FUNCTION_SET_DEBOUNCE_PERIOD, [debounce], 'L', 8, ''
    end

    # Returns the debounce period as set by BrickletTemperature#set_debounce_period.
    def get_debounce_period
      check_validity

      send_request FUNCTION_GET_DEBOUNCE_PERIOD, [], '', 12, 'L'
    end

    # Sets the I2C mode. Possible modes are:
    # 
    # * 0: Fast (400kHz)
    # * 1: Slow (100kHz)
    # 
    # If you have problems with obvious outliers in the
    # Temperature Bricklet measurements, they may be caused by EMI issues.
    # In this case it may be helpful to lower the I2C speed.
    # 
    # It is however not recommended to lower the I2C speed in applications where
    # a high throughput needs to be achieved.
    # 
    # .. versionadded:: 2.0.1$nbsp;(Plugin)
    def set_i2c_mode(mode)
      check_validity

      send_request FUNCTION_SET_I2C_MODE, [mode], 'C', 8, ''
    end

    # Returns the I2C mode as set by BrickletTemperature#set_i2c_mode.
    # 
    # .. versionadded:: 2.0.1$nbsp;(Plugin)
    def get_i2c_mode
      check_validity

      send_request FUNCTION_GET_I2C_MODE, [], '', 9, 'C'
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # The Raspberry Pi HAT (Zero) Brick is always at position 'i' and the Bricklet
    # connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always as
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

    # Registers a callback with ID <tt>id</tt> to the block <tt>block</tt>.
    def register_callback(id, &block)
      callback = block
      @registered_callbacks[id] = callback
    end
  end
end
