# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2020-05-19.      #
#                                                           #
# Ruby Bindings Version 2.1.26                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # Measures temperature with thermocouples
  class BrickletThermocouple < Device
    DEVICE_IDENTIFIER = 266 # :nodoc:
    DEVICE_DISPLAY_NAME = 'Thermocouple Bricklet' # :nodoc:

    # This callback is triggered periodically with the period that is set by
    # BrickletThermocouple#set_temperature_callback_period. The parameter is the
    # temperature of the thermocouple.
    # 
    # The CALLBACK_TEMPERATURE callback is only triggered if the temperature has
    # changed since the last triggering.
    CALLBACK_TEMPERATURE = 8

    # This callback is triggered when the threshold as set by
    # BrickletThermocouple#set_temperature_callback_threshold is reached.
    # The parameter is the temperature of the thermocouple.
    # 
    # If the threshold keeps being reached, the callback is triggered periodically
    # with the period as set by BrickletThermocouple#set_debounce_period.
    CALLBACK_TEMPERATURE_REACHED = 9

    # This Callback is triggered every time the error state changes
    # (see BrickletThermocouple#get_error_state).
    CALLBACK_ERROR_STATE = 13

    FUNCTION_GET_TEMPERATURE = 1 # :nodoc:
    FUNCTION_SET_TEMPERATURE_CALLBACK_PERIOD = 2 # :nodoc:
    FUNCTION_GET_TEMPERATURE_CALLBACK_PERIOD = 3 # :nodoc:
    FUNCTION_SET_TEMPERATURE_CALLBACK_THRESHOLD = 4 # :nodoc:
    FUNCTION_GET_TEMPERATURE_CALLBACK_THRESHOLD = 5 # :nodoc:
    FUNCTION_SET_DEBOUNCE_PERIOD = 6 # :nodoc:
    FUNCTION_GET_DEBOUNCE_PERIOD = 7 # :nodoc:
    FUNCTION_SET_CONFIGURATION = 10 # :nodoc:
    FUNCTION_GET_CONFIGURATION = 11 # :nodoc:
    FUNCTION_GET_ERROR_STATE = 12 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:

    THRESHOLD_OPTION_OFF = 'x' # :nodoc:
    THRESHOLD_OPTION_OUTSIDE = 'o' # :nodoc:
    THRESHOLD_OPTION_INSIDE = 'i' # :nodoc:
    THRESHOLD_OPTION_SMALLER = '<' # :nodoc:
    THRESHOLD_OPTION_GREATER = '>' # :nodoc:
    AVERAGING_1 = 1 # :nodoc:
    AVERAGING_2 = 2 # :nodoc:
    AVERAGING_4 = 4 # :nodoc:
    AVERAGING_8 = 8 # :nodoc:
    AVERAGING_16 = 16 # :nodoc:
    TYPE_B = 0 # :nodoc:
    TYPE_E = 1 # :nodoc:
    TYPE_J = 2 # :nodoc:
    TYPE_K = 3 # :nodoc:
    TYPE_N = 4 # :nodoc:
    TYPE_R = 5 # :nodoc:
    TYPE_S = 6 # :nodoc:
    TYPE_T = 7 # :nodoc:
    TYPE_G8 = 8 # :nodoc:
    TYPE_G32 = 9 # :nodoc:
    FILTER_OPTION_50HZ = 0 # :nodoc:
    FILTER_OPTION_60HZ = 1 # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 0]

      @response_expected[FUNCTION_GET_TEMPERATURE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_TEMPERATURE_CALLBACK_PERIOD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_TEMPERATURE_CALLBACK_PERIOD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_TEMPERATURE_CALLBACK_THRESHOLD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_TEMPERATURE_CALLBACK_THRESHOLD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_DEBOUNCE_PERIOD] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_GET_DEBOUNCE_PERIOD] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_CONFIGURATION] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_CONFIGURATION] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_ERROR_STATE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

      @callback_formats[CALLBACK_TEMPERATURE] = [12, 'l']
      @callback_formats[CALLBACK_TEMPERATURE_REACHED] = [12, 'l']
      @callback_formats[CALLBACK_ERROR_STATE] = [10, '? ?']

      @ipcon.add_device self
    end

    # Returns the temperature of the thermocouple.
    # 
    # If you want to get the temperature periodically, it is recommended
    # to use the CALLBACK_TEMPERATURE callback and set the period with
    # BrickletThermocouple#set_temperature_callback_period.
    def get_temperature
      check_validity

      send_request FUNCTION_GET_TEMPERATURE, [], '', 12, 'l'
    end

    # Sets the period with which the CALLBACK_TEMPERATURE callback is triggered
    # periodically. A value of 0 turns the callback off.
    # 
    # The CALLBACK_TEMPERATURE callback is only triggered if the temperature has changed
    # since the last triggering.
    def set_temperature_callback_period(period)
      check_validity

      send_request FUNCTION_SET_TEMPERATURE_CALLBACK_PERIOD, [period], 'L', 8, ''
    end

    # Returns the period as set by BrickletThermocouple#set_temperature_callback_period.
    def get_temperature_callback_period
      check_validity

      send_request FUNCTION_GET_TEMPERATURE_CALLBACK_PERIOD, [], '', 12, 'L'
    end

    # Sets the thresholds for the CALLBACK_TEMPERATURE_REACHED callback.
    # 
    # The following options are possible:
    # 
    #  "Option", "Description"
    # 
    #  "'x'",    "Callback is turned off"
    #  "'o'",    "Callback is triggered when the temperature is *outside* the min and max values"
    #  "'i'",    "Callback is triggered when the temperature is *inside* the min and max values"
    #  "'<'",    "Callback is triggered when the temperature is smaller than the min value (max is ignored)"
    #  "'>'",    "Callback is triggered when the temperature is greater than the min value (max is ignored)"
    def set_temperature_callback_threshold(option, min, max)
      check_validity

      send_request FUNCTION_SET_TEMPERATURE_CALLBACK_THRESHOLD, [option, min, max], 'k l l', 8, ''
    end

    # Returns the threshold as set by BrickletThermocouple#set_temperature_callback_threshold.
    def get_temperature_callback_threshold
      check_validity

      send_request FUNCTION_GET_TEMPERATURE_CALLBACK_THRESHOLD, [], '', 17, 'k l l'
    end

    # Sets the period with which the threshold callback
    # 
    # * CALLBACK_TEMPERATURE_REACHED
    # 
    # is triggered, if the threshold
    # 
    # * BrickletThermocouple#set_temperature_callback_threshold
    # 
    # keeps being reached.
    def set_debounce_period(debounce)
      check_validity

      send_request FUNCTION_SET_DEBOUNCE_PERIOD, [debounce], 'L', 8, ''
    end

    # Returns the debounce period as set by BrickletThermocouple#set_debounce_period.
    def get_debounce_period
      check_validity

      send_request FUNCTION_GET_DEBOUNCE_PERIOD, [], '', 12, 'L'
    end

    # You can configure averaging size, thermocouple type and frequency
    # filtering.
    # 
    # Available averaging sizes are 1, 2, 4, 8 and 16 samples.
    # 
    # As thermocouple type you can use B, E, J, K, N, R, S and T. If you have a
    # different thermocouple or a custom thermocouple you can also use
    # G8 and G32. With these types the returned value will not be in °C/100,
    # it will be calculated by the following formulas:
    # 
    # * G8: ``value = 8 * 1.6 * 2^17 * Vin``
    # * G32: ``value = 32 * 1.6 * 2^17 * Vin``
    # 
    # where Vin is the thermocouple input voltage.
    # 
    # The frequency filter can be either configured to 50Hz or to 60Hz. You should
    # configure it according to your utility frequency.
    # 
    # The conversion time depends on the averaging and filter configuration, it can
    # be calculated as follows:
    # 
    # * 60Hz: ``time = 82 + (samples - 1) * 16.67``
    # * 50Hz: ``time = 98 + (samples - 1) * 20``
    def set_configuration(averaging, thermocouple_type, filter)
      check_validity

      send_request FUNCTION_SET_CONFIGURATION, [averaging, thermocouple_type, filter], 'C C C', 8, ''
    end

    # Returns the configuration as set by BrickletThermocouple#set_configuration.
    def get_configuration
      check_validity

      send_request FUNCTION_GET_CONFIGURATION, [], '', 11, 'C C C'
    end

    # Returns the current error state. There are two possible errors:
    # 
    # * Over/Under Voltage and
    # * Open Circuit.
    # 
    # Over/Under Voltage happens for voltages below 0V or above 3.3V. In this case
    # it is very likely that your thermocouple is defective. An Open Circuit error
    # indicates that there is no thermocouple connected.
    # 
    # You can use the CALLBACK_ERROR_STATE callback to automatically get triggered
    # when the error state changes.
    def get_error_state
      check_validity

      send_request FUNCTION_GET_ERROR_STATE, [], '', 10, '? ?'
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # The Raspberry Pi HAT (Zero) Brick is always at position 'i' and the Bricklet
    # connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always as
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

    # Registers a callback with ID <tt>id</tt> to the block <tt>block</tt>.
    def register_callback(id, &block)
      callback = block
      @registered_callbacks[id] = callback
    end
  end
end
