# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2020-05-19.      #
#                                                           #
# Ruby Bindings Version 2.1.26                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

require_relative './ip_connection'

module Tinkerforge
  # Detects inclination of Bricklet (tilt switch open/closed)
  class BrickletTilt < Device
    DEVICE_IDENTIFIER = 239 # :nodoc:
    DEVICE_DISPLAY_NAME = 'Tilt Bricklet' # :nodoc:

    # This callback provides the current tilt state. It is called every time the
    # state changes.
    # 
    # See BrickletTilt#get_tilt_state for a description of the states.
    CALLBACK_TILT_STATE = 5

    FUNCTION_GET_TILT_STATE = 1 # :nodoc:
    FUNCTION_ENABLE_TILT_STATE_CALLBACK = 2 # :nodoc:
    FUNCTION_DISABLE_TILT_STATE_CALLBACK = 3 # :nodoc:
    FUNCTION_IS_TILT_STATE_CALLBACK_ENABLED = 4 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:

    TILT_STATE_CLOSED = 0 # :nodoc:
    TILT_STATE_OPEN = 1 # :nodoc:
    TILT_STATE_CLOSED_VIBRATING = 2 # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME

      @api_version = [2, 0, 0]

      @response_expected[FUNCTION_GET_TILT_STATE] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_ENABLE_TILT_STATE_CALLBACK] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_DISABLE_TILT_STATE_CALLBACK] = RESPONSE_EXPECTED_TRUE
      @response_expected[FUNCTION_IS_TILT_STATE_CALLBACK_ENABLED] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

      @callback_formats[CALLBACK_TILT_STATE] = [9, 'C']

      @ipcon.add_device self
    end

    # Returns the current tilt state. The state can either be
    # 
    # * 0 = Closed: The ball in the tilt switch closes the circuit.
    # * 1 = Open: The ball in the tilt switch does not close the circuit.
    # * 2 = Closed Vibrating: The tilt switch is in motion (rapid change between open and close).
    # 
    # .. image:: /Images/Bricklets/bricklet_tilt_mechanics.jpg
    #    :scale: 100 %
    #    :alt: Tilt states
    #    :align: center
    #    :target: ../../_images/Bricklets/bricklet_tilt_mechanics.jpg
    def get_tilt_state
      check_validity

      send_request FUNCTION_GET_TILT_STATE, [], '', 9, 'C'
    end

    # Enables the CALLBACK_TILT_STATE callback.
    def enable_tilt_state_callback
      check_validity

      send_request FUNCTION_ENABLE_TILT_STATE_CALLBACK, [], '', 8, ''
    end

    # Disables the CALLBACK_TILT_STATE callback.
    def disable_tilt_state_callback
      check_validity

      send_request FUNCTION_DISABLE_TILT_STATE_CALLBACK, [], '', 8, ''
    end

    # Returns *true* if the CALLBACK_TILT_STATE callback is enabled.
    def is_tilt_state_callback_enabled
      check_validity

      send_request FUNCTION_IS_TILT_STATE_CALLBACK_ENABLED, [], '', 9, '?'
    end

    # Returns the UID, the UID where the Bricklet is connected to,
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
    # The Raspberry Pi HAT (Zero) Brick is always at position 'i' and the Bricklet
    # connected to an :ref:`Isolator Bricklet <isolator_bricklet>` is always as
    # position 'z'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request FUNCTION_GET_IDENTITY, [], '', 33, 'Z8 Z8 k C3 C3 S'
    end

    # Registers a callback with ID <tt>id</tt> to the block <tt>block</tt>.
    def register_callback(id, &block)
      callback = block
      @registered_callbacks[id] = callback
    end
  end
end
